@@ -1,22 +1,6 @@
 /* SPDX-License-Identifier: LGPL-2.1+ */
 
-#ifndef _GNU_SOURCE
-#define _GNU_SOURCE
-#endif
-
-#include "../config.h"
-
-#ifdef HAVE_FUSE3
-#ifndef FUSE_USE_VERSION
-#define FUSE_USE_VERSION 30
-#endif
-#else
-#ifndef FUSE_USE_VERSION
-#define FUSE_USE_VERSION 26
-#endif
-#endif
-
-#define _FILE_OFFSET_BITS 64
+#include "config.h"
 
 #include <fcntl.h>
 #include <stdarg.h>
@@ -96,6 +80,21 @@ bool is_cgroup_fd(int fd)
 	return false;
 }
 
+bool is_cgroup2_fd(int fd)
+{
+	int ret;
+	struct statfs fs;
+
+	ret = fstatfs(fd, &fs);
+	if (ret)
+		return false;
+
+	if (is_fs_type(&fs, CGROUP2_SUPER_MAGIC))
+		return true;
+
+	return false;
+}
+
 void *must_realloc(void *orig, size_t sz)
 {
 	void *ret;
@@ -443,7 +442,7 @@ int safe_mount(const char *src, const char *dest, const char *fstype,
 	return 0;
 }
 
-#ifndef HAVE_STRLCPY
+#if !HAVE_STRLCPY
 size_t strlcpy(char *dest, const char *src, size_t size)
 {
 	size_t ret = strlen(src);
@@ -458,7 +457,7 @@ size_t strlcpy(char *dest, const char *src, size_t size)
 }
 #endif
 
-#ifndef HAVE_STRLCAT
+#if !HAVE_STRLCAT
 size_t strlcat(char *d, const char *s, size_t n)
 {
 	size_t l = strnlen(d, n);
@@ -782,11 +781,14 @@ int cgroup_walkup_to_root(int cgroup2_root_fd, int hierarchy_fd,
 		return 0;
 	}
 
+	if (!is_cgroup2_fd(dir_fd))
+		return -EINVAL;
+
 	/*
 	 * Legacy cgroup hierarchies should always show a valid value in the
 	 * file of the cgroup. So no need to do this upwards walking crap.
 	 */
-	if (cgroup2_root_fd < 0)
+	if (cgroup2_root_fd < 0 || !is_cgroup2_fd(cgroup2_root_fd))
 		return -EINVAL;
 	else if (same_file(cgroup2_root_fd, dir_fd))
 		return 1;
@@ -807,6 +809,9 @@ int cgroup_walkup_to_root(int cgroup2_root_fd, int hierarchy_fd,
 		if (dir_fd < 0)
 			return -errno;
 
+		if (!is_cgroup2_fd(dir_fd))
+			return log_error_errno(-ELOOP, ELOOP, "Found non-cgroup2 directory during cgroup2 tree walkup. Terminating walk");
+
 		/*
 		 * We're at the root of the cgroup2 tree so stop walking
 		 * upwards.
